# -*- coding: utf-8 -*-
from __future__ import absolute_import
import unittest
import tempfile
import mock
import socket
import os
import port_for
from port_for.utils import ranges_to_set


def test_common_ports():
    assert port_for.is_available(80) == False
    assert port_for.is_available(11211) == False


def test_good_port_ranges():
    ranges = [
        (10, 15), # too short
        (100, 200), # good
        (220, 245), # a bit short
        (300, 330), # good
        (440, 495), # also good
    ]

    ports = ranges_to_set(ranges)
    good_ranges = port_for.good_port_ranges(ports, 20, 3)
    assert good_ranges == [(103, 197), (443, 492), (303, 327)], good_ranges


def test_something_works():
    assert len(port_for.good_port_ranges()) > 10
    assert len(port_for.available_good_ports()) > 1000


def test_binding():
    # low ports are not available
    assert port_for.port_is_used(10) == True


def test_binding_high():
    s = socket.socket()
    s.bind(("", 0))
    port = s.getsockname()[1]
    assert port_for.port_is_used(port) == True
    s.close()
    assert port_for.port_is_used(port) == False


class SelectPortTest(unittest.TestCase):

    @mock.patch('port_for.api.port_is_used')
    def test_all_used(self, port_is_used):
        port_is_used.return_value = True
        self.assertRaises(port_for.PortForException, port_for.select_random)

    @mock.patch('port_for.api.port_is_used')
    def test_random_port(self, port_is_used):
        ports = set([1, 2, 3])
        used = {1: True, 2:False, 3:True}
        port_is_used.side_effect = lambda port: used[port]

        for x in range(100):
            self.assertEqual(port_for.select_random(ports), 2)


class StoreTest(unittest.TestCase):

    def setUp(self):
        fd, self.fname = tempfile.mkstemp()
        self.store = port_for.PortStore(self.fname)

    def tearDown(self):
        os.remove(self.fname)

    def test_store(self):
        assert self.store.bound_ports() == []

        port = self.store.bind_port('foo')
        self.assertTrue(port)
        self.assertEqual(self.store.bound_ports(), [('foo', port)])
        self.assertEqual(port, self.store.bind_port('foo'))

        port2 = self.store.bind_port('aar')
        self.assertNotEqual(port, port2)
        self.assertEqual(self.store.bound_ports(), [('foo', port), ('aar', port2)])

        self.store.unbind_port('aar')
        self.assertEqual(self.store.bound_ports(), [('foo', port)])

    def test_rebind(self):
        # try to rebind an used port for an another app
        port = self.store.bind_port('foo')
        self.assertRaises(port_for.PortForException, self.store.bind_port, 'baz', port)

    def test_change_port(self):
        # changing app ports is not supported.
        port = self.store.bind_port('foo')
        another_port = port_for.select_random()
        assert port != another_port
        self.assertRaises(port_for.PortForException, self.store.bind_port, 'foo', another_port)

    def test_bind_unavailable(self):
        # it is possible to explicitly bind currently unavailable port
        port = self.store.bind_port('foo', 80)
        self.assertEqual(port, 80)
        self.assertEqual(self.store.bound_ports(), [('foo', 80)])

    def test_bind_non_auto(self):
        # it is possible to pass a port
        port = port_for.select_random()
        res_port = self.store.bind_port('foo', port)
        self.assertEqual(res_port, port)



# -*- coding: utf-8 -*-
"""
This module/script is for updating port_for._ranges with recent information
from IANA and Wikipedia.
"""
from __future__ import absolute_import
import sys
import os
import re
import datetime
import urllib2
from xml.etree import ElementTree

name = os.path.abspath(os.path.normpath(os.path.join(os.path.dirname(__file__), '..')))
sys.path.insert(0, name)

from port_for.utils import to_ranges, ranges_to_set

IANA_DOWNLOAD_URL = 'https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xml'
IANA_NS = 'http://www.iana.org/assignments'
WIKIPEDIA_PAGE = 'http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers'


def _write_unassigned_ranges(out_filename):
    """
    Downloads ports data from IANA & Wikipedia and converts
    it to a python module. This function is used to generate _ranges.py.
    """
    with open(out_filename, 'wt') as f:
        f.write('# auto-generated by port_for._download_ranges (%s)\n' % datetime.date.today())
        f.write('UNASSIGNED_RANGES = [\n')
        for range in to_ranges(sorted(list(_unassigned_ports()))):
            f.write("    (%d, %d),\n" % range)
        f.write(']\n')


def _unassigned_ports():
    """ Returns a set of all unassigned ports (according to IANA and Wikipedia) """
    free_ports = ranges_to_set(_parse_ranges(_iana_unassigned_port_ranges()))
    known_ports = ranges_to_set(_wikipedia_known_port_ranges())
    return free_ports.difference(known_ports)


def _wikipedia_known_port_ranges():
    """
    Returns used port ranges according to Wikipedia page.
    This page contains unofficial well-known ports.
    """
    req = urllib2.Request(WIKIPEDIA_PAGE, headers={'User-Agent' : "Magic Browser"})
    page = urllib2.urlopen(req).read().decode('utf8')

    # just find all numbers in table cells
    ports = re.findall('<td>((\d+)(\W(\d+))?)</td>', page, re.U)
    return ((int(p[1]), int(p[3] if p[3] else p[1])) for p in ports)


def _iana_unassigned_port_ranges():
    """
    Returns unassigned port ranges according to IANA.
    """
    page = urllib2.urlopen(IANA_DOWNLOAD_URL).read()
    xml = ElementTree.fromstring(page)
    records = xml.findall('{%s}record' % IANA_NS)
    for record in records:
        description = record.find('{%s}description' % IANA_NS).text
        if description == 'Unassigned':
            numbers = record.find('{%s}number' % IANA_NS).text
            yield numbers


def _parse_ranges(ranges):
    """ Converts a list of string ranges to a list of [low, high] tuples. """
    for txt in ranges:
        if '-' in txt:
            low, high = txt.split('-')
        else:
            low, high = txt, txt
        yield int(low), int(high)


if __name__ == '__main__':
    _write_unassigned_ranges('_ranges.py')



#!/usr/bin/env python
from setuptools import setup


setup(
    name='port-for',
    version='0.4',
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['port_for'],
    scripts=['scripts/port-for'],

    url='https://github.com/kmike/port-for/',
    license='MIT license',
    description="""Utility that helps with local TCP ports managment. It can find an unused TCP localhost port and remember the association.""",

    long_description = open('README.rst').read(),

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',

        'Operating System :: POSIX',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Systems Administration',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
)
